package dev.flyfish.framework.query;

import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.query.chain.DefaultLambdaQueryChain;
import dev.flyfish.framework.query.chain.DefaultNamedQueryChain;
import dev.flyfish.framework.query.support.DomainFunction;
import org.springframework.data.repository.core.EntityInformation;

/**
 * 查询工具类
 * 基于Fluent API风格实现
 * 底层基于适配器兼容各种数据库,包括关系型数据库和
 * 自动判定空值，空值默认情况不会进行查询
 *
 * @author wangyu
 */
public final class Queries {

    /**
     * 查询转换
     *
     * @param query             通用查询
     * @param entityInformation 实体信息
     * @param <T>               具体查询泛型
     * @return 转换结果
     */
    @SuppressWarnings("unchecked")
    public static <T> T convert(Query query, EntityInformation<? extends Domain, String> entityInformation) {
        return QueryFactories.getFactory(entityInformation)
                .map(factory -> (T) factory.getConverter().convert(query))
                .orElse(null);
    }

    /**
     * 创建基于字符串字段名的查询
     *
     * @param column 列名
     * @return 结果
     */
    public static QueryCondition<NamedQueryChain> where(String column) {
        NamedQueryChain chain = new DefaultNamedQueryChain();
        return chain.and(column);
    }

    /**
     * 创建基于lambda字段引用的查询
     *
     * @param getter 列引用
     * @return 结果
     */
    public static <T extends Domain> QueryCondition<LambdaQueryChain<T>> where(DomainFunction<T> getter) {
        LambdaQueryChain<T> chain = new DefaultLambdaQueryChain<>();
        return chain.and(getter);
    }

    /**
     * 以嵌套条件开始
     *
     * @param chain 要嵌套的条件
     * @return 结果
     */
    public static NamedQueryChain within(NamedQueryChain chain) {
        NamedQueryChain created = new DefaultNamedQueryChain();
        return created.and(chain);
    }

    /**
     * 以嵌套条件开始
     *
     * @param chain 要嵌套的条件
     * @return 结果
     */
    public static <T extends Domain> LambdaQueryChain<T> within(LambdaQueryChain<T> chain) {
        LambdaQueryChain<T> created = new DefaultLambdaQueryChain<>();
        return created.and(chain);
    }

    /**
     * 传入多个查询列表时的连接方式
     */
    public enum Combinator {

        AND, OR
    }

    /**
     * 方向，用于描述查询和部分语法
     */
    public enum Direction {

        LEFT, RIGHT, ALL
    }
}
