package dev.flyfish.framework.utils;

import lombok.extern.slf4j.Slf4j;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 日期区间工具类
 *
 * @author wybab
 */
@Slf4j
public final class DateRangeUtil {

    /**
     * 获取日期区间
     *
     * @param start 开始日期
     * @param end   截止日期
     * @param weeks 日期
     * @return 结果
     */
    public static Map<Byte, List<Date>> getDates(Date start, Date end, Set<Byte> weeks) {
        // 获取日历
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(start);
        // 构建结果集
        Map<Byte, List<Date>> resultMap = new HashMap<>();
        Date current;
        // 开始循环查找
        while (!(current = calendar.getTime()).after(end)) {
            // 获得周数字
            byte week = (byte) calendar.get(Calendar.DAY_OF_WEEK);
            if (weeks.contains(week)) {
                List<Date> group = resultMap.computeIfAbsent(week, key -> new ArrayList<>());
                group.add(current);
            }
            // 增加
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }
        return resultMap;
    }

    /**
     * 获取指定日期的开始时间
     *
     * @param date 日期
     * @return 日期的开始时间
     */
    public static Date getBeginTimeOfDate(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        // 取当天的开始时间
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        return calendar.getTime();
    }

    /**
     * 获取指定日期的结束时间
     *
     * @param date 日期
     * @return 日期的结束时间
     */
    public static Date getEndTimeOfDate(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        // 取当天的结束时间
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);

        return calendar.getTime();
    }

    /**
     * 获取时间段内所有的年月集合
     *
     * @param minDate 最小时间  2017-01
     * @param maxDate 最大时间 2017-10
     * @return 日期集合 格式为 年-月
     */
    public static List<String> getMonthBetween(String minDate, String maxDate) {
        try {
            ArrayList<String> result = new ArrayList<>();
            SimpleDateFormat month = new SimpleDateFormat("yyyy-MM");//格式化为年月

            Calendar min = Calendar.getInstance();
            Calendar max = Calendar.getInstance();

            min.setTime(month.parse(minDate));
            min.set(min.get(Calendar.YEAR), min.get(Calendar.MONTH), 1);

            max.setTime(month.parse(maxDate));
            max.set(max.get(Calendar.YEAR), max.get(Calendar.MONTH), 2);

            while (min.before(max)) {
                result.add(month.format(min.getTime()));
                min.add(Calendar.MONTH, 1);
            }
            return result;
        } catch (ParseException e) {
            log.error("日期格式错误！", e);
            return new ArrayList<>();
        }
    }

    public static List<String> getRangeOfMonth(String month) {
        try {
            ArrayList<String> result = new ArrayList<>();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");

            Calendar cal = Calendar.getInstance();
            cal.setTime(sdf.parse(month));

            sdf.applyPattern("yyyy-MM-dd");
            cal.set(Calendar.DAY_OF_MONTH, 1);
            result.add(sdf.format(getBeginTimeOfDate(cal.getTime())));
            cal.set(Calendar.DAY_OF_MONTH, cal.getActualMaximum(Calendar.DAY_OF_MONTH));
            result.add(sdf.format(getEndTimeOfDate(cal.getTime())));

            return result;
        } catch (ParseException e) {
            log.error("日期格式错误！", e);
            return new ArrayList<>();
        }
    }

}
