package dev.flyfish.framework.file.service;


import dev.flyfish.framework.file.config.UploadConfiguration;
import dev.flyfish.framework.file.domain.Media;
import dev.flyfish.framework.file.enums.MediaType;
import dev.flyfish.framework.file.utils.FileSizeUtils;
import dev.flyfish.framework.file.utils.ZipUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.codec.multipart.FilePart;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import jakarta.annotation.Resource;
import java.io.IOException;

/**
 * 媒体服务
 *
 * @author wangyu
 */
@Service
@Slf4j
public class MediaService {

    private static String URL = "/api/medias/";
    @Resource
    private FileService fileService;
    @Resource
    private UploadConfiguration uploadConfiguration;

    /**
     * 上传媒体文件
     *
     * @param part 文件
     * @return 结果
     */
    public Mono<Media> upload(FilePart part) {
        return fileService.saveLocal(part)
                .map(path -> Media.builder()
                        .size(FileSizeUtils.size(part.headers().getContentLength()))
                        .type(MediaType.accept(part.filename()))
                        .url(URL + path)
                        .build()
                );
    }

    /**
     * 分析原媒体文件，尝试解压
     *
     * @param part 原文件
     * @return 结果
     */
    public Mono<Media> slice(FilePart part) {
        String localPath = uploadConfiguration.getLocalPath();
        return fileService.saveLocal(part)
                .flatMap(path -> {
                    try {
                        return ZipUtils.unzip(localPath + path)
                                .stream().filter(file -> file.endsWith(".m3u8"))
                                .map(file -> file.replace(localPath, ""))
                                .findFirst()
                                .map(Mono::just)
                                .orElse(Mono.just(path));
                    } catch (IOException e) {
                        log.error("很不幸，解压发生了错误：", e);
                    }
                    return Mono.just(path);
                })
                .map(path -> Media.builder()
                        .size(FileSizeUtils.size(part.headers().getContentLength()))
                        .type(MediaType.accept(part.filename()))
                        .url(URL + path)
                        .build()
                );
    }

    /**
     * 上传到绝对路径
     *
     * @param part         文件
     * @param absolutePath 绝对路径
     * @return 结果
     */
    public Mono<Media> upload(FilePart part, String absolutePath) {
        Mono<String> saver = StringUtils.isNotBlank(absolutePath) ? fileService.saveLocalPath(part, absolutePath) :
                fileService.saveLocal(part);
        return saver.map(path -> Media.builder()
                .size(FileSizeUtils.size(part.headers().getContentLength()))
                .type(MediaType.accept(part.filename()))
                .url(URL + path)
                .build()
        );
    }
}
