package dev.flyfish.framework.user.config;

import com.anji.captcha.config.AjCaptchaAutoConfiguration;
import com.anji.captcha.config.AjCaptchaServiceAutoConfiguration;
import com.anji.captcha.config.AjCaptchaStorageAutoConfiguration;
import com.anji.captcha.properties.AjCaptchaProperties;
import dev.flyfish.framework.configuration.jwt.JwtProperties;
import dev.flyfish.framework.configuration.jwt.JwtSecurityContextRepository;
import dev.flyfish.framework.configuration.jwt.TokenProvider;
import dev.flyfish.framework.domain.po.User;
import dev.flyfish.framework.enums.UserStatus;
import dev.flyfish.framework.enums.UserType;
import dev.flyfish.framework.handler.JsonAuthenticationFailureHandler;
import dev.flyfish.framework.handler.JsonAuthenticationSuccessHandler;
import dev.flyfish.framework.handler.JsonLogoutSuccessHandler;
import dev.flyfish.framework.service.AuthenticationAuditor;
import dev.flyfish.framework.service.AuthenticationLogger;
import dev.flyfish.framework.user.config.captcha.CaptchaValidator;
import dev.flyfish.framework.user.config.converter.EncryptedAuthenticationConverter;
import dev.flyfish.framework.user.config.properties.SecurityProperties;
import dev.flyfish.framework.user.domain.UserQo;
import dev.flyfish.framework.user.initializer.UserInitializer;
import dev.flyfish.framework.user.service.UserService;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.ReactiveAuthenticationManager;
import org.springframework.security.authentication.UserDetailsRepositoryReactiveAuthenticationManager;
import org.springframework.security.config.annotation.web.reactive.EnableWebFluxSecurity;
import org.springframework.security.config.web.server.SecurityWebFiltersOrder;
import org.springframework.security.config.web.server.ServerHttpSecurity;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.ReactiveUserDetailsService;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.factory.PasswordEncoderFactories;
import org.springframework.security.crypto.password.DelegatingPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.web.server.SecurityWebFilterChain;
import org.springframework.security.web.server.authentication.AuthenticationWebFilter;
import org.springframework.security.web.server.authentication.HttpStatusServerEntryPoint;
import org.springframework.security.web.server.authentication.ServerAuthenticationConverter;
import org.springframework.security.web.server.authorization.HttpStatusServerAccessDeniedHandler;
import org.springframework.security.web.server.context.ServerSecurityContextRepository;
import org.springframework.security.web.server.context.WebSessionServerSecurityContextRepository;
import reactor.core.publisher.Mono;

import static org.springframework.security.web.server.util.matcher.ServerWebExchangeMatchers.pathMatchers;

/**
 * @author wangyu
 */
@EnableWebFluxSecurity
@Order(1)
@EnableAutoConfiguration(exclude = AjCaptchaAutoConfiguration.class)
@EnableConfigurationProperties({JwtProperties.class, SecurityProperties.class})
@Import(WebSecurityConfig.AjCaptchaAutoConfiguration.class)
public class WebSecurityConfig {

    /**
     * 设置密码加密策略
     *
     * @return 密码加密器
     */
    @Bean
    public static PasswordEncoder passwordEncoder() {
        DelegatingPasswordEncoder delegatingPasswordEncoder =
                (DelegatingPasswordEncoder) PasswordEncoderFactories.createDelegatingPasswordEncoder();
        //设置defaultPasswordEncoderForMatches为NoOpPasswordEncoder
        delegatingPasswordEncoder.setDefaultPasswordEncoderForMatches(new BCryptPasswordEncoder());
        return delegatingPasswordEncoder;
    }

    @Bean
    public AuthenticationAuditor loginAuditor() {
        return new AuthenticationAuditorImpl();
    }

    @Bean
    @ConditionalOnMissingBean(AuthenticationLogger.class)
    public AuthenticationLogger authenticationLogger() {
        return new AuthenticationLogger() {
            @Override
            public Mono<Void> success(UserDetails user) {
                return Mono.empty();
            }

            @Override
            public Mono<Void> failure(User user, AuthenticationException exception) {
                return Mono.empty();
            }

            @Override
            public Mono<Void> logout(UserDetails user) {
                return Mono.empty();
            }
        };
    }

    @ConditionalOnProperty(value = "jwt.enable", havingValue = "true")
    @Bean("contextRepository")
    public JwtSecurityContextRepository jwtSecurityContextRepository() {
        return new JwtSecurityContextRepository();
    }

    @ConditionalOnProperty(value = "jwt.enable", havingValue = "false")
    @Bean
    public ServerSecurityContextRepository contextRepository() {
        return new WebSessionServerSecurityContextRepository();
    }

    @Bean
    public TokenProvider tokenProvider(JwtProperties properties) {
        return new TokenProvider(properties);
    }

    /**
     * spring安全拦截规则配置
     *
     * @param http            配置器
     * @param dataTransformer 数据转换器
     * @return 结果
     */
    @Bean
    @SuppressWarnings("all")
    public SecurityWebFilterChain springSecurityFilterChain(ServerHttpSecurity http,
                                                            TokenProvider tokenProvider,
                                                            SecurityProperties properties,
                                                            ReactiveUserDetailsService userDetailsService,
                                                            ServerAuthenticationConverter authenticationConverter,
                                                            AuthenticationAuditor authenticationAuditor) {
        ReactiveAuthenticationManager authenticationManager = new UserDetailsRepositoryReactiveAuthenticationManager(userDetailsService);
        return http
                .securityContextRepository(contextRepository())
                .authorizeExchange(spec -> spec.pathMatchers(ArrayUtils.addAll(properties.getAllowUris(), "/api/logout", "/api/login")).permitAll()
                        .pathMatchers("/api/users/**").authenticated()
                        .anyExchange().authenticated())
                .formLogin(spec -> spec.disable()) // 配置登录节点
                .httpBasic(spec -> spec.disable())
                .logout(spec -> spec
                        .logoutUrl("/api/logout")
                        .logoutSuccessHandler(new JsonLogoutSuccessHandler(authenticationAuditor, tokenProvider)))
                .exceptionHandling(spec -> spec
                        .authenticationEntryPoint(new HttpStatusServerEntryPoint(HttpStatus.UNAUTHORIZED))
                        .accessDeniedHandler(new HttpStatusServerAccessDeniedHandler(HttpStatus.UNAUTHORIZED)))
                .csrf(spec -> spec.disable())
                .addFilterAt(
                        configure(authenticationManager, authenticationAuditor, authenticationConverter),
                        SecurityWebFiltersOrder.FORM_LOGIN)
                .build();
    }

    @Bean
    public ServerAuthenticationConverter encryptedAuthenticateConverter(SecurityProperties securityProperties,
                                                                        ObjectProvider<CaptchaValidator> validator) {
        return new EncryptedAuthenticationConverter(securityProperties, validator);
    }

    /**
     * 默认的初始化
     *
     * @param userService 用户服务
     * @return 结果
     */
    @Bean
    @ConditionalOnMissingBean(UserInitializer.class)
    @SuppressWarnings("all")
    public UserInitializer userInitializer(UserService userService) {
        return () -> {
            UserQo qo = new UserQo();
            qo.setType(UserType.SUPER_ADMIN.name());
            userService.count(qo)
                    .filter(count -> count == 0)
                    .flatMap(count -> {
                        // 初始化用户
                        User user = new User();
                        user.setUsername("admin");
                        user.setPassword("Sxu@Unicom#2021");
                        user.setType(UserType.SUPER_ADMIN);
                        user.setEnable(true);
                        user.setApp(false);
                        user.setStatus(UserStatus.NORMAL);
                        user.setPhone("10000000000");
                        user.setName("超级管理员");
                        user.setCode("Administrator");
                        return userService.createSelective(user);
                    }).subscribe();
        };
    }

    /**
     * 配置登录相关参数
     *
     * @param authenticationAuditor   审计器
     * @param authenticationConverter 转换器
     * @param authenticationManager   鉴权管理器
     * @return 结果
     */
    private AuthenticationWebFilter configure(ReactiveAuthenticationManager authenticationManager,
                                              AuthenticationAuditor authenticationAuditor,
                                              ServerAuthenticationConverter authenticationConverter) {
        AuthenticationWebFilter authenticationFilter = new AuthenticationWebFilter(authenticationManager);
        authenticationFilter.setRequiresAuthenticationMatcher(pathMatchers(HttpMethod.POST, "/login", "/api/login"));
        authenticationFilter.setAuthenticationFailureHandler(new JsonAuthenticationFailureHandler(authenticationAuditor));
        authenticationFilter.setServerAuthenticationConverter(authenticationConverter);
        authenticationFilter.setAuthenticationSuccessHandler(new JsonAuthenticationSuccessHandler(authenticationAuditor));
        authenticationFilter.setSecurityContextRepository(contextRepository());
        return authenticationFilter;
    }

    @Configuration
    @EnableConfigurationProperties({AjCaptchaProperties.class})
    @ConditionalOnProperty(value = "security.captcha.enable", havingValue = "true")
    @Import({AjCaptchaServiceAutoConfiguration.class, AjCaptchaStorageAutoConfiguration.class})
    static class AjCaptchaAutoConfiguration {
    }
}
